/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: dft.h,v 1.4 2002/05/08 21:24:47 jdejong Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jdejong 2002-05-08  imported
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/* $Id: dft.h,v 1.4 2002/05/08 21:24:47 jdejong Exp $ */

/*-----------------------------------------------------------------------
 *  Discrete Fourier transform package.
 *  This package performs discrete Fourier Transforms of unequally
 *  spaced data.  Applications involving regularly spaced data should
 *  us an FFT which will generally be MUCH faster.
 *     A structure containing the sines/cosines is created once and then
 *  can be used multiple times.  
 *  Complex numbers are stored as real and imaginary parts in successive 
 *  array elements.
 *
 *  The Fourier Transforms included are:
 *     DFTc2c  - Complex to complex.
 *     DFTr2c  - Real to complex.
 *     DFTc2r  - Complex to real.
 *  To use this package:
 *   1) Create and initialize an DFT_info using makeDFT
 *   2) Call the appropriate routine FT routine
 *   3) Free the memory used by the DFT_info using killDFT
 *---------------------------------------------------------------------*/
#include "VLTItable.h"
#ifndef DFT_H
#define DFT_H
/* define forward and backward transforms */
enum DFT_dir {FORWARD=1, BACKWARD=-1};

/* data structure for sine/cosine values, etc. needed for efficiency */
typedef struct DFT_INFO {
  long num_in;        /* number of values in input array */
  long num_out;       /* number of values in output array */
  float *cos_arr;     /* array of cosines */
  float *sin_arr;     /* array of sines */
} DFT_info;

DFT_info* makeDFT (int dir, long num_x, float* x, long num_u, float* u);
/*----------------------------------------------------------------------- */
/*   Make an DFT_info structure and fill values.                          */
/*   This structure needs to be created for a specific Fourier transform  */
/*   although once created may be used for as many repetitions of the same*/
/*   transform as are needed.                                             */
/*   The terms "X" and "U" refer to the initial and transformed domains.  */
/*   Note: x and u arrays should be in appropriate physical units and NOT */
/*   merely indices.                                                      */
/*   Inputs:                                                              */
/*       dir   The direction of the transform: FORWARD or BACKWARD        */
/*       num_x The number of elements in the X direction                  */
/*             This is the number of complex elements if x is complex     */
/*       x     Array of coordinate values on X axis                       */
/*       num_u The number of elements in the U direction                  */
/*             This is the number of complex elements if u is complex     */
/*       u     Array of coordinate values on U axis                       */
/*   Returns:                                                             */
/*      An DFT_info structure pointer ; NULL indicates failure            */
/*----------------------------------------------------------------------- */

void killDFT (DFT_info* DFi);
/*----------------------------------------------------------------------- */
/*   Delete an DFT_info structure and free memory                         */
/*   Inputs:                                                              */
/*       DFi   The structure to be deleted                                */
/*----------------------------------------------------------------------- */

void DFTc2c (DFT_info* DFi, float in[], float out[]);
/*----------------------------------------------------------------------- */
/*   Perform Complex to Complex Fourier transform                         */
/*   The direction, input and output coordinate values are specified when */
/*   the DFT_info DFi was created.                                        */
/*   Inputs:                                                              */
/*       DFi   The structure defining the transform, created by makeDFT   */
/*       in    Input array of values as described to makeDFT              */
/*             complex values are represented as real, imag in successive */
/*             elements of the array                                      */
/*   Outputs:                                                             */
/*       out   Output array of values as described to MakeDFT             */
/*             complex values are represented as real, imag in successive */
/*             elements of the array                                      */
/*----------------------------------------------------------------------- */

void DFTr2c (DFT_info* DFi, float in[], float out[]);
/*----------------------------------------------------------------------- */
/*   Perform Real to Complex Fourier transform                            */
/*   The direction, input and output coordinate values are specified when */
/*   the DFT_info DFi was created.                                        */
/*   Inputs:                                                              */
/*       DFi   The structure defining the transform, created by makeDFT   */
/*       in    Input array of real values as described to makeDFT         */
/*   Outputs:                                                             */
/*       out   Output array of values as described to MakeDFT             */
/*             complex values are represented as real, imag in successive */
/*             elements of the array                                      */
/*----------------------------------------------------------------------- */

void DFTc2r (DFT_info* DFi, float in[], float out[]);
/*----------------------------------------------------------------------- */
/*   Perform Complex to Complex Fourier transform                         */
/*   The direction, input and output coordinate values are specified when */
/*   the DFT_info DFi was created.                                        */
/*   Inputs:                                                              */
/*       DFi   The structure defining the transform, created by makeDFT   */
/*       in    Input array of values as described to makeDFT              */
/*             complex values are represented as real, imag in successive */
/*             elements of the array                                      */
/*   Outputs:                                                             */
/*       out   Output array of real values as described to MakeDFT        */
/*             complex values are represented as real, imag in successive */
/*             elements of the array                                      */
/*----------------------------------------------------------------------- */

#endif  /* DFT_H */
/* */
